<?php
/*
Plugin Name: Logical Captcha
Plugin URI: http://maccessibility.net
Description: This plugin takes advantage of the TextCaptcha.com service, which provides logic-based textual questions instead of distorted images or audio to validate that the entity registering is a real live human being, and not a spam bot. Textual logic-based captchas mean that your registration process will be accessible to everyone, including visually/hearing impaired visitors who use access technology to surf the web. It requires a free API key from TextCaptcha.com
Author: Josh de Lioncourt
Version: 1.0.3
Author URI: http://maccessibility.net
*/

/*  Copyright 2009 the Mac-cessibility Network

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/

if ( !class_exists( 'LogicalCaptcha' ) )
{ // if

	class LogicalCaptcha
	{ // class

		// Constructor
		function LogicalCaptcha()
		{ // function
			add_action('register_form',array(&$this,'add_logical_captcha_fields'));
			add_action('register_post',array(&$this,'register_post'),10,2);
			add_action('admin_menu',array(&$this,'logical_captcha_menu'));
		} // function

		// Add fields to the form
		function add_logical_captcha_fields()
		{ // function
			$text_captcha_url = "http://textcaptcha.com/api/";
			$lc_api_key_value = get_option('lc_api_key');

			// load captcha using web service
			$url = $text_captcha_url . $lc_api_key_value;
			try
			{ // try
				$xml = @new SimpleXMLElement($url,null,true);
			} catch (Exception $e) {
				// if there is a problem, use static fallback..
				$fallback = '<captcha>' .
				'<question>Out of a rose, novel, dog, table, piano, or automobile, which is most likely to have teeth?</question>' .
				'<answer>'.md5('dog').'</answer></captcha>';
				$xml = new SimpleXMLElement($fallback);
			} // try
			// display question
			$question = (string) $xml->question;
			echo "<div><p>You must answer the following question to proceed.</p>\n";
			echo "<p>$question</p>";
			echo "<p><input type=\"text\" name=\"logical_answer\" title=\"The Answer\" size=\"20\" />";
			// extract possible answers
			$ans = array();
			foreach ($xml->answer as $hash)
			{ //foreach
				$ans[] = (string) $hash;
			} // foreach
			// store the answer in the session
			$_SESSION['captcha'] = $ans;
		} // function

		//  Registration Protection
		function register_post($user_login, $user_email)
		{ // function
			if (($user_login != '') && ($user_email != ''))
			{ // if
				$this->check_logic();
			} // if
		} // function

		// Evaluate Captcha
		function check_logic()
		{ // function
			$user_ans = $_POST['logical_answer'];
			$user_ans = strtolower(trim($user_ans));
			$user_ans = md5($user_ans);
			if (!in_array($user_ans,$_SESSION['captcha']))
			{ // if
				$response = "<h1>Highly Illogical</h1>";
				$response .= "<p>You failed to answer the question to confirm you are a human. Please <a href=\"";
				$response .= $_SERVER['HTTP_REFERER'];
				$response .= "\">click here to try again</a>.</p>";
				wp_die($response, '403 Forbidden', array( 'response' => 403 ) );
			} // if
		} // function

		function logical_captcha_menu()
		{ // function
			add_options_page('Logical Captcha', 'Logical Captcha', 8, 'logical_captcha',array(&$this,'logical_captcha_options'));
		} // function

		function logical_captcha_options()
		{ // function
			// variables for the field and option names 
			$lc_api_key = 'lc_api_key';
			$lc_submitted = 'lc_submitted';
			// Read in existing option value from database
			$lc_api_key_value = get_option($lc_api_key);
			// See if the user has posted information
			// If they did, this hidden field will be set to 'Y'
			if ($_POST[$lc_submitted] == 'Y')
			{ // if
				// Read their posted value
				$lc_api_key_value = $_POST[$lc_api_key];
				// Save the posted value in the database
				update_option( $lc_api_key, $lc_api_key_value);
				// Put an options updated message on the screen
				?>
				<div class="updated"><p><strong>API key saved.</strong></p></div>
				<?php
			} // if
			// Now display the options editing screen
			echo '<div class="wrap">';
			// header
			echo "<h2>Logical Captcha</h2>";
			// options form
			?>
			<form name="settings" method="post" action="<?php echo str_replace('%7E','~',$_SERVER['REQUEST_URI']);?>">
			<input type="hidden" name="<?php echo $lc_submitted;?>" value="Y">
			<p>The <b>Logical Captcha</b> plugin requires that you be registered with <a href="http://textcaptcha.com" target="_blank">TextCaptcha.com</a>, which serves us logic-based textual 	captcha question and answer pairs. When you register with <b>TextCaptcha</b>, you will be given an API key, with which you can access the service. Enter the key you were provided here.</p>
			<p><input type="text" name="<?php echo $lc_api_key;?>" title="API Key" value="<?php echo $lc_api_key_value;?>" size="40" /></p>
			<hr />
			<p class="submit"><input type="submit" name="Submit" value="Save"></p>
			</form>
			</div>
			<?php
		} // function

	} // class

	$wp_logical_captcha = new LogicalCaptcha();
} // if

?>